import scipy as sp
from matplotlib.pyplot import *

##################################
def main():
    iht = IHT()
    ist = IST()
    tst = TST()
    lars = LARS()
    omp = OMP()
    l1 = L_1_Theo()
    figure(1, figsize=(12,10))
    clf()
    axes = gca()
    font = {'size'   : 25}
    matplotlib.rc('font', **font)
    plot(iht[:,0], iht[:,1],'g-x')
    plot(ist[:,0], ist[:,1],'c-x')
    plot(tst[:,0], tst[:,1],'r-x')
    plot(lars[:,0], lars[:,1],'m-x')
    plot(omp[:,0], omp[:,1],'k-x')
    plot(l1[:,0], l1[:,1],'b-')
    setp(axes.lines, linewidth=5)
    setp(axes.lines, markersize=20)
    setp(axes.lines, markeredgewidth=5)
    xlabel(r"$\delta$")
    ylabel(r"$\rho$")
    legend(('IHT','IST','TST','LARS','OMP','L1 Theoretical'), loc=2)

##################################
def IHT():
    IHT = sp.array(
            [[0.0500,   0.1028],
             [0.0828,   0.1342],
             [0.1155,   0.1544],
             [0.1483,   0.1628],
             [0.1810,   0.1781],
             [0.2138,   0.1976],
             [0.2466,   0.2168],
             [0.2793,   0.2168],
             [0.3121,   0.2250],
             [0.3448,   0.2413],
             [0.3776,   0.2495],
             [0.4103,   0.2558],
             [0.4431,   0.2685],
             [0.4759,   0.2802],
             [0.5086,   0.2865],
             [0.5414,   0.2910],
             [0.5741,   0.3055],
             [0.6069,   0.3127],
             [0.6397,   0.3235],
             [0.6724,   0.3334],
             [0.7052,   0.3425],
             [0.7379,   0.3461],
             [0.7707,   0.3569],
             [0.8034,   0.3695],
             [0.8362,   0.3867],
             [0.8690,   0.3976],
             [0.9017,   0.4048],
             [0.9345,   0.4139],
             [0.9672,   0.4238],
             [1.0000,   0.4375]])
    return IHT

##################################
def IST():
    IST = sp.array(
            [[0.0500,   0.0938],
             [0.0828,   0.1272],
             [0.1155,   0.1391],
             [0.1483,   0.1557],
             [0.1810,   0.1624],
             [0.2138,   0.1801],
             [0.2466,   0.1756],
             [0.2793,   0.1903],
             [0.3121,   0.1987],
             [0.3448,   0.2087],
             [0.3776,   0.2177],
             [0.4103,   0.2214],
             [0.4431,   0.2287],
             [0.4759,   0.2332],
             [0.5086,   0.2440],
             [0.5414,   0.2495],
             [0.5741,   0.2513],
             [0.6069,   0.2621],
             [0.6397,   0.2612],
             [0.6724,   0.2648],
             [0.7052,   0.2775],
             [0.7379,   0.2847],
             [0.7707,   0.2883],
             [0.8034,   0.2973],
             [0.8362,   0.2973],
             [0.8690,   0.3064],
             [0.9017,   0.3091],
             [0.9345,   0.3208],
             [0.9672,   0.3325],
             [1.0000,   0.3497]])   
    return IST

##################################
def TST():
    TST = sp.array(
            [[0.0500,   0.1247],
             [0.0828,   0.1530],
             [0.1155,   0.1705],
             [0.1483,   0.1904],
             [0.1810,   0.2034],
             [0.2138,   0.2192],
             [0.2466,   0.2302],
             [0.2793,   0.2486],
             [0.3121,   0.2623],
             [0.3448,   0.2779],
             [0.3776,   0.2916],
             [0.4103,   0.2999],
             [0.4431,   0.3108],
             [0.4759,   0.3191],
             [0.5086,   0.3374],
             [0.5414,   0.3365],
             [0.5741,   0.3511],
             [0.6069,   0.3685],
             [0.6397,   0.3840],
             [0.6724,   0.3858],
             [0.7052,   0.3996],
             [0.7379,   0.4133],
             [0.7707,   0.4270],
             [0.8034,   0.4453],
             [0.8362,   0.4499],
             [0.8690,   0.4617],
             [0.9017,   0.4791],
             [0.9345,   0.4883],
             [0.9672,   0.5075],
             [1.0000,   0.8000]]) 
    return TST

##################################
def LARS():
    LARS = sp.array(
            [[0.0500,   0.1370],
             [0.0810,   0.1804],
             [0.1121,   0.1891],
             [0.1431,   0.2081],
             [0.1741,   0.2218],
             [0.2052,   0.2418],
             [0.2362,   0.2541],
             [0.2672,   0.2665],
             [0.2983,   0.2813],
             [0.3293,   0.2912],
             [0.3603,   0.3047],
             [0.3914,   0.3158],
             [0.4224,   0.3207],
             [0.4534,   0.3453],
             [0.4845,   0.3625],
             [0.5155,   0.3724],
             [0.5466,   0.3834],
             [0.5776,   0.4031],
             [0.6086,   0.4105],
             [0.6397,   0.4240],
             [0.6707,   0.4400],
             [0.7017,   0.4572],
             [0.7328,   0.4756],
             [0.7638,   0.4953],
             [0.7948,   0.5041],
             [0.8259,   0.5409],
             [0.8569,   0.5681],
             [0.8879,   0.5898],
             [0.9190,   0.6184],
             [0.9500,   0.6850]])
    return LARS

##################################
def OMP():
    OMP = sp.array(
            [[0.0500,   0.1117],
             [0.0828,   0.1476],
             [0.1155,   0.1542],
             [0.1483,   0.1725],
             [0.1810,   0.1792],
             [0.2138,   0.1922],
             [0.2466,   0.2077],
             [0.2793,   0.2187],
             [0.3121,   0.2159],
             [0.3448,   0.2332],
             [0.3776,   0.2422],
             [0.4103,   0.2513],
             [0.4431,   0.2585],
             [0.4759,   0.2675],
             [0.5086,   0.2856],
             [0.5414,   0.2910],
             [0.5741,   0.3000],
             [0.6069,   0.3154],
             [0.6397,   0.3280],
             [0.6724,   0.3416],
             [0.7052,   0.3515],
             [0.7379,   0.3732],
             [0.7707,   0.3849],
             [0.8034,   0.4075],
             [0.8362,   0.4230],
             [0.8690,   0.4568],
             [0.9017,   0.4848],
             [0.9345,   0.5291],
             [0.9672,   0.6000],
             [1.0000,   0.6000]])
    return OMP

##################################
def L_1_Theo():
    L_1_Theo = sp.array(
            [[0.0500,   0.1540],
             [0.0744,   0.1724],
             [0.0987,   0.1883],
             [0.1231,   0.2020],
             [0.1474,   0.2154],
             [0.1718,   0.2282],
             [0.1962,   0.2404],
             [0.2205,   0.2523],
             [0.2449,   0.2640],
             [0.2692,   0.2755],
             [0.2936,   0.2869],
             [0.3179,   0.2982],
             [0.3423,   0.3095],
             [0.3667,   0.3208],
             [0.3910,   0.3322],
             [0.4154,   0.3437],
             [0.4397,   0.3553],
             [0.4641,   0.3671],
             [0.4885,   0.3791],
             [0.5128,   0.3913],
             [0.5372,   0.4037],
             [0.5615,   0.4165],
             [0.5859,   0.4297],
             [0.6103,   0.4433],
             [0.6346,   0.4573],
             [0.6590,   0.4719],
             [0.6833,   0.4871],
             [0.7077,   0.5031],
             [0.7321,   0.5198],
             [0.7564,   0.5376],
             [0.7808,   0.5565],
             [0.8051,   0.5768],
             [0.8295,   0.5988],
             [0.8538,   0.6230],
             [0.8782,   0.6500],
             [0.9026,   0.6807],
             [0.9269,   0.7168],
             [0.9513,   0.7619],
             [0.9756,   0.8243],
             [1.0000,   0.9324]]) 
    return L_1_Theo           

