import scipy as sp
import scipy.special as sps
import sys
import cPickle
from matplotlib.pyplot import *

def display(fileName, doublekappa=False):
    # Load results - use try/catch construction to support legacy output
    try:
        filePtr = open(fileName, 'rb')
        (settings, activeRecvs, SNRs_dB, \
            berSim, noBits) = cPickle.load(filePtr)
    except ValueError:
        filePtr = open(fileName, 'rb')
        (settings, activeRecvs, SNRs_dB, \
            dmy, berSim, noBits) = cPickle.load(filePtr)
    filePtr.close()
    if doublekappa:
        fileName2 = "saved_results/m10_kappa4.pkl"
        try:
            filePtr2 = open(fileName2, 'rb')
            (settings2, activeRecvs2, SNRs_dB2, \
                berSim2, noBits2) = cPickle.load(filePtr2)
        except ValueError:
            filePtr2 = open(fileName2, 'rb')
            (settings2, activeRecvs2, SNRs_dB2, \
                dmy, berSim2, noBits2) = cPickle.load(filePtr2)

    M = 4
    noncoherent_berTheo = noncoherent_BERvsSNR(sp.log2(settings['N']+1))

    # Display results
    figure(1, figsize=(16,13))
    clf()
    font = {'size'   : 25}
    matplotlib.rc('font', **font)
    markers = ('x', '+', 's', '*', '<')
    colors = ('r', 'g', 'c', 'm', 'y')
    for jj in xrange(activeRecvs.size):
        plot(SNRs_dB, berSim[:,jj],colors[jj]+'-'+markers[jj])
    if doublekappa:
        for jj in xrange(1,activeRecvs2.size):
            plot(SNRs_dB2, berSim2[:,jj],colors[jj]+'--'+markers[jj])
    plot(SNRs_dB, noncoherent_berTheo,'b-.o')
    axes = gca()
    axes.set_yscale('log')
    matplotlib.rc('font', **font)
    xlim(SNRs_dB[0]-1, SNRs_dB[-1]+1)
    if doublekappa:
        xlim(SNRs_dB[0]-1, SNRs_dB2[-1]+1)
    ylim(10**-5, 10**0)
    xlabel('SNR [dB]')
    ylabel('BER')
    setp(axes.lines, linewidth=5)
    setp(axes.lines, markersize=20)
    setp(axes.lines, markeredgewidth=5)
    if doublekappa:
        legend(('Classical Recv.','CS Recv. - $\kappa=2$','RS CS Recv. - $\kappa=2$', 
            'RD CS Recv. - $\kappa=2$','CSS Recv. - $\kappa=2$',
            'CS Recv. - $\kappa=4$','RD CS Recv. - $\kappa=4$',
            'CSS Recv. - $\kappa=4$','Theoretical'), loc="lower left")
    else:
        legend(('Classical Recv.','CS Recv.','RS CS Recv.', 
            'RD CS Recv.','CSS Recv.','Theoretical'), loc="lower left")
    grid()
    draw()
    show()

def noncoherent_BERvsSNR(m):
    berTheo = 0
    if m == 5:
        berTheo = sp.array([.21645723, 0.05559462151, 0.001877863929, 8.683255707*10**(-7), 1.092664008*10**(-13)])
    if m == 7:
        berTheo = sp.array([0.2993716543, 0.102309449844, 0.004998911619, 0.0000030177817, 3.7527623208536*10**(-13)])
    if m == 10:
        berTheo = sp.array([0.193315728107, 0.016527929959, 0.000017326203, 2.37772657050*10**(-12)])
    return berTheo

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("You must input a filename!")

    fileName = sys.argv[1]
    display(fileName)


