from __future__ import division
import scipy as sp
import sys
import cPickle
from matplotlib.pyplot import *
from matplotlib.mlab import find

def display():
    filenames = sp.array([  'saved_results/cs_m10.pkl', 
                            'saved_results/rdcs_m10.pkl',
                            'saved_results/csscs_m10.pkl'])
    colors = sp.array(['g','m','y'])

    clf()
    for ii in xrange(filenames.size):
        # Load results
        filePtr = open(filenames[ii], 'rb')
        (settings, convergence, success, subpur_iters, proc_times) = cPickle.load(filePtr)
        filePtr.close()
        convergence = sp.array(convergence)
        gridSize = settings['gridSize']

        # Display results
        font = {'size'   : 25}
        matplotlib.rc('font', **font)
        fig = figure(1, figsize=(9,7.5))
        matplotlib.rc('font', **font)
        deltas = sp.arange(1,gridSize+1,dtype=float)/gridSize
        rhos = deltas
        line = sp.ones((gridSize,1))
        for jj in xrange(gridSize-1): # Must be gridSize-1 because last column has no transition line - it always succeeds
            delta_col = success[:,jj]
            rho_idx = find(abs(delta_col-0.5) == min(abs(delta_col-0.5)))[0]
            line[jj] = rhos[rho_idx]
        plot(deltas, line, colors[ii])

    # Display Maleki paper data
    delta = sp.linspace(0.05,1,30)
    rho = sp.array([0.1247, 0.1530, 0.1705, 0.1904, 0.2034, 0.2192, 0.2302,\
                        0.2486, 0.2623, 0.2779, 0.2916, 0.2999, 0.3108, 0.3191,\
                        0.3374, 0.3365, 0.3511, 0.3685, 0.3840, 0.3858, 0.3996,\
                        0.4133, 0.4270, 0.4453, 0.4499, 0.4617, 0.4791, 0.4883,\
                        0.5075, 0.8000])
    plot(delta, rho, 'k', hold=True)        
    axes = gca()
    setp(axes.lines, linewidth=4)
    setp(axes.lines, markersize=20)
    setp(axes.lines, markeredgewidth=2)

    xlabel(r"$\delta=M/N$")
    ylabel(r"$\rho=S/M$")
    legend(('Bernoulli','RD','CSS','Tuned TST'),loc='upper left')
    grid()
    draw()
    show()

if __name__ == "__main__":
    display()


