from __future__ import division
import scipy as sp
import sys
import cPickle
from matplotlib.pyplot import *

def display(fileName, malekiline=False):
    # Load results
    filePtr = open(fileName, 'rb')
    (settings, convergence, success, subpur_iters, proc_times) = cPickle.load(filePtr)
    convergence = sp.array(convergence)
    filePtr.close()
    gridSize = settings['gridSize']

    # Maleki paper data
    delta = sp.linspace(0.05,1,30)
    rho = sp.array([0.1247, 0.1530, 0.1705, 0.1904, 0.2034, 0.2192, 0.2302,\
                        0.2486, 0.2623, 0.2779, 0.2916, 0.2999, 0.3108, 0.3191,\
                        0.3374, 0.3365, 0.3511, 0.3685, 0.3840, 0.3858, 0.3996,\
                        0.4133, 0.4270, 0.4453, 0.4499, 0.4617, 0.4791, 0.4883,\
                        0.5075, 0.8000])

    # Display results
    font = {'size'   : 25}
    matplotlib.rc('font', **font)
    fig = figure(1, figsize=(9,7.5))
    clf()
    matplotlib.rc('font', **font)
    deltas = sp.arange(1,gridSize+1,dtype=float)/gridSize
    rhos = deltas
    (X, Y) = sp.meshgrid(deltas, rhos)
    contourf(X, Y, success, cmap=cm.jet_r)
    colorbar()
    xlabel(r"$\delta=M/N$")
    ylabel(r"$\rho=S/M$")
    clim(0,1)
    grid()

    # Display Maleki paper data, if so chosen
    if malekiline:
        plot(delta, rho, 'w-x', hold=True)        
        axes = gca()
        setp(axes.lines, linewidth=4)
        setp(axes.lines, markersize=20)
        setp(axes.lines, markeredgewidth=2)

    draw()
    show()

    # Display convergence plot
    figure(2)
    clf()
    plot(sp.arange(convergence.size)+1, convergence)
    axes = gca()
    axes.set_yscale('log')
    xlabel("Experiement no.")
    ylabel("MSE")
    setp(axes.lines, linewidth=2)

    # Display Subspace Pursuit iteration plot
    font = {'size'   : 25}
    matplotlib.rc('font', **font)
    fig = figure(3, figsize=(9,7.5))
    clf()
    matplotlib.rc('font', **font)
    deltas = sp.arange(1,gridSize+1,dtype=float)/gridSize
    rhos = deltas
    (X, Y) = sp.meshgrid(deltas, rhos)
    ticks = sp.arange(1,subpur_iters.max()+1)
    contourf(X, Y, subpur_iters, cmap=cm.jet, levels=ticks)
    colorbar(ticks=ticks)
    xlabel(r"$\delta=M/N$")
    ylabel(r"$\rho=S/M$")
    grid()

    # Display computational cost plot
    font = {'size'   : 25}
    matplotlib.rc('font', **font)
    fig = figure(4, figsize=(16,7.5))
    clf()
    matplotlib.rc('font', **font)
    deltas = sp.arange(1,gridSize+1,dtype=float)/gridSize
    rhos = deltas
    (X, Y) = sp.meshgrid(deltas, rhos)
    subplot(121)
    contourf(X, Y, proc_times, cmap=cm.jet)
    colorbar()
    xlabel(r"$\delta=M/N$")
    ylabel(r"$\rho=S/M$")
    grid()
    subplot(122)
    N = settings['N']
    M = sp.int_(sp.round_(deltas*N))
    (X, Y) = sp.meshgrid(deltas, rhos)
    Z = sp.zeros((gridSize, gridSize))
    c = 3*10**9
    for ii in xrange(gridSize):
        for jj in xrange(gridSize):
            K = subpur_iters[ii,jj]
            S = sp.int_(sp.round_(M[jj]*rhos[ii]))
            Z[ii,jj] = 99*K*S**3 + 4*(K+1)*M[jj]*N + 2*(K+1)*M[jj] + 4*(K+2)*M[jj]*S + 10*K*M[jj]*S**2 + c*K
    ticks = sp.array([0.0, 0.15, 0.30, 0.45, 0.60, 0.75, 0.90, 1.0])
    contourf(X, Y, Z/Z.max(), cmap=cm.jet, levels=ticks)
    colorbar(ticks=ticks)
    xlabel(r"$\delta=M/N$")
    ylabel(r"$\rho=S/M$")
    grid()

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("You must input a filename!")

    fileName = sys.argv[1]
    display(fileName)


