# IMPORTS ########################################################### IMPORTS #
# General Python imports
from __future__ import division
import scipy as sp
import cPickle
import multiprocessing as mp
from time import time
from matplotlib.mlab import find
import scipy.signal as sps
import scipy.linalg as spl

# PyComm imports
import pycomm.modulation.psk as psk
import pycomm.generators.rrcos as rrcos
import pycomm.RF as RF
import pycomm.sampling.rrcosmf as rrcosmf
import pycomm.quantizers.uniform as quantizerUniform
import pycomm.auxiliary.berc as berc
import pycomm.reconstruction.subpur as subpur

# CLASSRECV_BER_TEST ###################################### CLASSRECV_BER_TEST #
def classrecv_ber_test(settings, DicMtx):
    N = settings.get('N')
    quanBits = settings.get('quanBits')
    MeasMtx = sp.eye(N)
    return ber_test(MeasMtx, DicMtx, settings, quanBits)

# CSRECV_BER_TEST ########################################### CSRECV_BER_TEST #
def csrecv_ber_test(settings, DicMtx):
    N = settings.get('N')
    kappa = settings.get('kappa')
    quanBits = kappa*settings.get('quanBits')
    O = sp.int_(sp.round_(N/kappa))
    if O < 1:
        O = 1
    MeasMtx = 2*sp.random.randint(0,2,size=(O,N))-1
    return ber_test(MeasMtx, DicMtx, settings, quanBits)

# RDCSRECV_DTPT_TEST ##################################### RDCSRECV_DTPT_TEST #
def rdcsrecv_ber_test(settings, DicMtx):
    N = settings.get('N')
    kappa = settings.get('kappa')
    quanBits = kappa*settings.get('quanBits')
    O = sp.int_(sp.round_(N/kappa))
    if O < 1:
        O = 1
    MeasVec = 2*sp.random.randint(0,2,N)-1
    MeasMtx = sp.zeros((O,N))
    ONratio = sp.int_(sp.floor(N/O))
    if ONratio < 1:
        ONratio = 1
    idx1 = 0; idx2 = 0
    for ii in xrange(O):
        idx1 = idx2
        if ii < sp.mod(N,O):
            idx2 = idx1+ONratio+1
            colIdx = sp.int_(sp.linspace(idx1, idx2, ONratio+1, endpoint=False))
        else:
            idx2 = idx1+ONratio
            colIdx = sp.int_(sp.linspace(idx1, idx2, ONratio, endpoint=False))
        MeasMtx[ii,colIdx] = MeasVec[colIdx]
    return ber_test(MeasMtx, DicMtx, settings, quanBits)

# CSSCSRECV_DTPT_TEST ##################################### CSSCSRECV_DTPT_TEST #
def csscsrecv_ber_test(settings, DicMtx):
    N = settings.get('N')
    kappa = settings.get('kappa')
    quanBits = kappa*settings.get('quanBits')
    O = sp.int_(sp.round_(N/kappa))
    if O < 1:
        O = 1
    MeasVec = sp.ones(N)
    MeasMtx = sp.zeros((O,N))
    ONratio = sp.int_(sp.floor(N/O))
    if ONratio < 1:
        ONratio = 1
    idx1 = 0; idx2 = 0
    for ii in xrange(O):
        idx1 = idx2
        if ii < sp.mod(N,O):
            idx2 = idx1+ONratio+1
            colIdx = sp.int_(sp.linspace(idx1, idx2, ONratio+1, endpoint=False))
        else:
            idx2 = idx1+ONratio
            colIdx = sp.int_(sp.linspace(idx1, idx2, ONratio, endpoint=False))
        MeasMtx[ii,colIdx] = MeasVec[colIdx]
    return ber_test(MeasMtx, DicMtx, settings, quanBits)

# BER_TEST ######################################################### BER_TEST #
def ber_test(MeasMtx, DicMtx, settings, quanBits=0):
    # Experiment settings
    EbN0_dB = settings.get('EbN0')
    modulationOrder = settings.get('modulationOrder')
    noRuns = settings.get('noRuns')
    noSymbols = settings.get('noSymbolsPerRun')
    N = settings.get('N')   # Number of Nyquist samples
    S = settings.get('S')   # Density, i.e. how many codes are in one signal
    noBits = sp.log2(modulationOrder)*S*noSymbols
    f_chip = 10**6 # 1MHz - approx. GPS chip freq.
    f_sl = 10*f_chip
    f_c = 3*f_chip
    f_sRF = 4*f_c
    T_chip = 1./f_chip
    T_sl = 1./f_sl
    T_sRF = 1./f_sRF

    # Seed random generator
    if sp.size(mp.current_process()._identity)>0:
        procID = mp.current_process()._identity[0]
        sp.random.seed( int(1e8 * ( time()-int(time()) ) * procID) )
    
    errors = 0
    for ii in xrange(noRuns):
        # Generate bit vector
        bits = sp.random.randint(0,2,noBits)

        # Generate baseband CDMA signal
        (t, s) = generateSignal(bits, modulationOrder, N, S, 
                                    noSymbols, DicMtx, T_chip, T_sl)
        Duration = t[-1]
        
        # Generate RF signal
        t_RF = sp.linspace(0, Duration, Duration/T_sRF, endpoint=False) 
        s_ups = sps.resample(s, t_RF.size)
        s_RF = RF.upconv(s_ups, f_c, f_sRF)
    
        # Add AWGN
        s_RF_noisy = RF.awgn(s_RF, EbN0_dB, f_sRF, noBits)
        
        # Down convert
        s_hat_ups = RF.downconv(s_RF_noisy, f_c, f_sRF, 0, 0, f_chip)
        s_hat = sps.resample(s_hat_ups, t.size)

        bits_hat = sp.zeros((noSymbols, sp.log2(modulationOrder)*S))
        for jj in xrange(noSymbols):
            symbIdx = find( (t >= (jj*T_chip*N)) & (t < ((jj+2)*T_chip*N)) )
            symbSignal = s_hat[symbIdx]

            # Vectorize symbSignal and sample
            symbSignal_vec = sp.zeros((symbSignal.size,1), complex)
            symbSignal_vec[:,0] = symbSignal
            symbSamples = sampleSignal(symbSignal_vec, MeasMtx, DicMtx, N, T_chip, T_sl)

            # Quantize
            if quanBits > 0:
                mini = -1.5*S; maxi = 1.5*S
                symbSamples = quantizerUniform(symbSamples, quanBits, mini, maxi)
            # Pre-whitening
            C = spl.cholesky(sp.dot(MeasMtx, MeasMtx.T), lower=True)
            symbSamples = spl.solve(C, symbSamples)
            newMeasMtx = spl.solve(C, MeasMtx)

            # Receiver structure
            symbBits = demodulateSignal(symbSamples, newMeasMtx, DicMtx, N,
                                        S, modulationOrder)
            bits_hat[jj,:] = symbBits

        bits_hat = sp.int_(bits_hat.reshape(bits.shape))

        # Calculate BER
        (ber, errors_ii) = berc(bits, bits_hat)
        errors += errors_ii
    return errors

# GENERATESIGNAL ############################################# GENERATESIGNAL #
def generateSignal(bits, modulationOrder, N, S, noSymbols, Dictionary, T_chip, T_sl):
    # Generate symbol vector
    (tmp, Constellation) = psk.mod(bits, modulationOrder, 0, S)

    # Randomly permute Alpha
    alpha = sp.zeros((N,tmp.shape[1]), complex)
    for ii in xrange(tmp.shape[1]):
        randomIdx = sp.sort(sp.random.permutation(sp.arange(0,N))[0:S])
        alpha[randomIdx,ii] = tmp[:,ii]

    # Apply CDMA modulation
    s_d = sp.dot(Dictionary, alpha)
    
    # Generate time vector
    StartDelay = T_chip
    EndDelay = T_chip
    Duration = StartDelay + N*noSymbols*T_chip
    t = sp.linspace(0, Duration, Duration/T_sl,endpoint=False) 

    # Generate continous signal using pulse shape function rrcos
    s_d_vec = s_d.T.reshape((s_d.shape[0]*s_d.shape[1]))
    pulseWindow = 10
    s = rrcos(t, T_chip, 1., StartDelay, s_d_vec, pulseWindow)

    return t, s

# SAMPLESIGNAL ################################################# SAMPLESIGNAL #
def sampleSignal(s_hat, MeasMtx, DicMtx, N, T_chip, T_sl):
    # Generate root raised cosine pulses
    (dmy, SampMtx) = rrcosmf(s_hat, N, T_chip, T_sl)

    # Construct receiver specific sampling matrix
    recvSampMtx = sp.dot(MeasMtx, SampMtx)

    # Sample input signal and normalize
    samples = sp.dot(recvSampMtx, s_hat)*T_sl
    samples = samples/T_chip # normalization
    return samples

# DEMODULATESIGNAL ######################################### DEMODULATESIGNAL #
def demodulateSignal(s_d, MeasMtx, DicMtx, N, S, modulationOrder):
    # Generate A matrix
    A = sp.dot(MeasMtx, DicMtx)

    # Demodulate CDMA
    (x_hat, dmy1, dmy2) = subpur(s_d, A, S, False)

    # Identify significant codes and extract them
    threshold = sp.sort(abs(x_hat),0)[-S]
    alpha_hat = x_hat[find(abs(x_hat)>=threshold)][0:S]

    # Demodulate PSK
    bits_hat = psk.demod(alpha_hat, modulationOrder)
    return bits_hat
