import scipy as sp
import scipy.special as sps
import sys
import cPickle
from matplotlib.pyplot import *

def display(filename, quanExp=False):
    # Load results - use try/catch construction to support legacy output
    try:
        filePtr = open(filename, 'rb')
        (settings, activeRecvs, EbN0s_dB, \
            berSim, noBits) = cPickle.load(filePtr)
    except ValueError:
        filePtr = open(filename, 'rb')
        (settings, activeRecvs, EbN0s_dB, \
            dmy, berSim, noBits) = cPickle.load(filePtr)
    filePtr.close()

    if quanExp:
        # Load secondary results
        quanFilename = 'saved_results/quan_rf_m7_kappa2_quan4.pkl'
        try:
            filePtr = open(quanFilename, 'rb')
            (settings2, activeRecvs2, EbN0s_dB2, \
                berSim2, noBits2) = cPickle.load(filePtr)
        except ValueError:
            filePtr = open(quanFilename, 'rb')
            (settings2, activeRecvs2, EbN0s_dB2, \
                dmy, berSim2, noBits2) = cPickle.load(filePtr)
        filePtr.close()

    noncoherent_berTheo = noncoherent_BERvsEbN0(sp.log2(settings['N']+1))

    # Display results
    figure(1, figsize=(13,13))
    clf()
    font = {'size'   : 25}
    matplotlib.rc('font', **font)
    if quanExp:
        markers = ('x', '<', 'x')
        colors = ('r', 'y', 'r')
    else:
        markers = ('x', '+', '*', '<')
        colors = ('r', 'g', 'm', 'y')
        
    for jj in xrange(activeRecvs.size):
        plot(EbN0s_dB, berSim[:,jj],colors[jj]+'-'+markers[jj])
    if quanExp:
        plot(EbN0s_dB, berSim2[:,0],colors[activeRecvs.size]+'--'+markers[activeRecvs.size])
    if not(quanExp):
        plot(EbN0s_dB, noncoherent_berTheo,'b-.o')
    axes = gca()
    axes.set_yscale('log')
    matplotlib.rc('font', **font)
    xlim(EbN0s_dB[0]-1, EbN0s_dB[-1]+1)
    ylim(10**-5, 10**0)
    xlabel('Eb/N0 [dB]')
    ylabel('BER')
    setp(axes.lines, linewidth=5)
    setp(axes.lines, markersize=20)
    setp(axes.lines, markeredgewidth=5)
    if quanExp:
        legend(('Classical Recv. - Quan 2','CSS Recv. - Quan 4', 'Classical Recv. - Quan 4'), loc="lower left")
    else:
        legend(('Classical Recv.','CS Recv.', 
            'RD CS Recv.','CSS Recv.','Theoretical'), loc="lower left")
    grid()
    draw()
    show()

def noncoherent_BERvsEbN0(m):
    berTheo = 0
    if m == 5:
        berTheo = sp.array([0.218728935807181, 0.057196493669806, 0.002011793887946, 0.0000010082577067, 1.4770030176134*10**(-13)])
    if m == 7:
        berTheo = sp.array([0.302947319010127, 0.1060405722488525, 0.005500106915160770, 0.00000376392660944, 5.903605960046736*10**(-13)])
    if m == 10:
        berTheo = sp.array([0.400033897116, 0.2010190931, 0.018701038798, 0.000023811026678, 4.69861239768729*10**(-12)])
    return berTheo

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("You must input a filename!")

    filename = sys.argv[1]
    display(filename)


